<?php

namespace Drupal\lb_plus\Controller;

use Drupal\Core\Url;
use Drupal\layout_builder\Section;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Ajax\AjaxHelperTrait;
use Drupal\Core\Ajax\CloseDialogCommand;
use Drupal\layout_builder\SectionStorageInterface;
use Drupal\Core\Layout\LayoutPluginManagerInterface;
use Drupal\layout_builder\LayoutBuilderHighlightTrait;
use Drupal\layout_builder\Controller\LayoutRebuildTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\layout_builder\Context\LayoutBuilderContextTrait;
use Drupal\layout_builder\LayoutTempstoreRepositoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;

class ChangeSectionLayout implements ContainerInjectionInterface {

  use AjaxHelperTrait;
  use LayoutRebuildTrait;
  use StringTranslationTrait;
  use LayoutBuilderContextTrait;
  use LayoutBuilderHighlightTrait;

  protected LayoutPluginManagerInterface $layoutManager;
  protected LayoutTempstoreRepositoryInterface $layoutTempstoreRepository;

  public function __construct(LayoutTempstoreRepositoryInterface $tempstore_repository, LayoutPluginManagerInterface $layout_manager) {
    $this->layoutTempstoreRepository = $tempstore_repository;
    $this->layoutManager = $layout_manager;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('layout_builder.tempstore_repository'),
      $container->get('plugin.manager.core.layout')
    );
  }

  /**
   * Choose a layout plugin for this section.
   *
   * @param \Drupal\layout_builder\SectionStorageInterface $section_storage
   *   The section storage.
   * @param int $delta
   *   The delta of the section to splice.
   *
   * @return array
   *   The render array.
   */
  public function chooseLayout(SectionStorageInterface $section_storage, int $delta) {
    $items = [];
    $definitions = $this->layoutManager->getFilteredDefinitions('layout_builder', $this->getPopulatedContexts($section_storage), ['section_storage' => $section_storage]);
    foreach ($definitions as $plugin_id => $definition) {
      $item = [
        '#type' => 'link',
        '#title' => [
          'icon' => $definition->getIcon(60, 80, 1, 3),
          'label' => [
            '#type' => 'container',
            '#children' => $definition->getLabel(),
          ],
        ],
        '#url' => Url::fromRoute('lb_plus.configure_changed_layout', [
          'section_storage_type' => $section_storage->getStorageType(),
          'section_storage' => $section_storage->getStorageId(),
          'delta' => $delta,
          'plugin_id' => $plugin_id,
        ]),
      ];
      if ($this->isAjax()) {
        $item['#attributes']['class'][] = 'use-ajax';
        $item['#attributes']['data-dialog-type'][] = 'dialog';
        $item['#attributes']['data-dialog-renderer'][] = 'off_canvas';
      }
      $items[$plugin_id] = $item;
    }
    $output['layouts'] = [
      '#theme' => 'item_list__layouts',
      '#items' => $items,
      '#attributes' => [
        'class' => [
          'layout-selection',
        ],
        'data-layout-builder-target-highlight-id' => $this->sectionAddHighlightId($delta),
      ],
    ];

    return $output;
  }

  /**
   * Change and configure new layout.
   *
   * @param \Drupal\layout_builder\SectionStorageInterface $section_storage
   *   The section storage.
   * @param int $delta
   *   The section delta.
   * @param string $plugin_id
   *   The new layout plugin ID.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   Saves and displays the section in the new layout, closes the layout
   *   options dialog, and then opens the section configuration modal by clicking
   *   the link for the user.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function changeAndConfigureNewLayout(SectionStorageInterface $section_storage, int $delta, string $plugin_id) {
    $section = $section_storage->getSection($delta);
    // Recreate the section in order to change it's layout.
    $layout_plugin = $this->layoutManager->createInstance($plugin_id);
    $third_party_settings = [];
    foreach ($section->getThirdPartyProviders() as $provider) {
      $third_party_settings[$provider] = $section->getThirdPartySettings($provider);
    }
    // Place the blocks in the main region.
    $default_region = $layout_plugin->getPluginDefinition()->getDefaultRegion();
    $components = $section->getComponents();
    foreach ($components as $component) {
      $component->setRegion($default_region);
    }
    $new_section = new Section($plugin_id, $layout_plugin->getConfiguration(), $components, $third_party_settings);
    $section_storage->insertSection($delta, $new_section);
    $section_storage->removeSection($delta + 1);

    $this->layoutTempstoreRepository->set($section_storage);

    $response = $this->rebuildLayout($section_storage);
    $response->addCommand(new CloseDialogCommand('.ui-dialog-content'));
    $uuid = $new_section->getThirdPartySetting('lb_plus', 'uuid');
    $response->addCommand(new InvokeCommand('', 'LBPlusConfigureSection', [$uuid]));
    return $response;
  }

}
